# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
package Sitalibs::Web::Bulk;

use Exporter;

our @ISA = qw ( Exporter );

our @EXPORT = qw ( &new );
our @EXPORT_OK = qw ( &new );

use strict;
use fields;
use Sitalibs::Common::Logger;

my $DEBUG=1;


sub new {
    my ($class,@colonnes) = @_;
    my $this = {};
    $this->{KEYS} = \@colonnes;
    $this->{TMPFILE} = "/tmp/upload_feed.tmp";
    $this->{FRMFILE} = "Web/feed_form.cfg";
    $this->{TPLFILE} = '';
    $this->{DELIMITOR} = { none => '', quote => "\'", dblquote => "\""};
		$this->{SEP_LIST} = { space => ' ', tabul => "\t", virg => ",", pt_virg=>";"}; 
    $this->{SEPARATOR} = ',';
    $this->{TEXT} = '';
    $this->{UPFILE} ='';
    $this->{SUCCESS} = 0;
    bless($this,$class);
    return $this;
}

####################################################################
# sub IsSuccess
# paramtre de retour : la valeur de SUCCESS
####################################################################
sub IsSuccess()
{
    my ($this) = @_;
    return($this->{SUCCESS});
}

####################################################################
# sub Template
#
# Sert d'accesseur pour le champ TPLFILE
#
# Parametre :
# optionnel $tpl : le nom du template
#
# Retour : le nom du template
####################################################################
sub Template($,$)
{
    my ($this,$tpl) = @_;
    debug("settemplate $tpl\n") if ($DEBUG);
    $this->{TPLFILE} = $tpl;
    return($this->{TPLFILE});
}

####################################################################
# sub Form
#
#
#
# Parametre :
# optionnel $tpl : le template
#
# Retour : la page a afficher
####################################################################
sub Form($,$)
{
    my ($this,$tpl) = @_;
    my $page = '';
    if(conduse("Sitalibs::Web::Forms")
       && conduse("Sitalibs::Common::TemplateParse",qw(parse))
       && conduse("Sitalibs::Common::TemplateParse"))
    {
				my $form_struct = form_read($this->{FRMFILE}, 'feedupload');
				my $form = form_print_parts($form_struct);
				$form->{FEED_KEYS} = join(', ',@{$this->{KEYS}});
				if($this->{TPLFILE} eq '')
				{
						debug("tpl string\n") if ($DEBUG);
						$page = parse($tpl,$form);
				}
				else
				{
						debug("tpl file $this->{TPL}\n") if ($DEBUG);
						$page = remplace($this->{TPL},$form);
				}
    }
    else
    {
				print STDERR "erreur chargement sitalibs\n";
				$page = "erreur chargement sitalibs";
    }
    return($page);
}

# renvoie 0 si erreur
# sinon la taille du fichier uploader en K octet
sub Upload ($$,$$)
{
    my ($this,$q,$tmp_file,$mxsize) = @_;
    my $max_size;
		if(!defined($tmp_file))
    {
				$this->{TMPFILE} = "/tmp/upload_feed.tmp";
    }
		if(!defined($mxsize))
		{# taille maximal par defaut en nombre de groupe
				$max_size = 20;
		}
		else
		{
				$max_size = $mxsize;
		}
    my $fh;
    $this->{UPFILE} = $q->param('data');
    debug("filename = $this->{UPFILE}\n") if ($DEBUG);
    my $br; # nombre de groupe d'octets lus
		my $n_oct = 1024; # un groupe correspond a n_oct
    my $size = 0;
    my $str = '';
    my $buffer;
    if(!open($fh,">$this->{TMPFILE}"))
    {
				error("erreur d ouverture file $tmp_file\n");
				return(0);
    }
    # lecture du fichier uploader et criture du fichier temporaire
    while($br = read( $this->{UPFILE}, $buffer, $n_oct) && $size<$max_size)
    {
				debug("upload br = $br\n") if ($DEBUG);
				$size += $br;
				$str = $str.$buffer;
				binmode $fh;
				print $fh $buffer;
    }
    close($fh);
    debug("size uploaded = $size ko\n") if ($DEBUG);
		if($size>=$max_size)
		{
				$max_size = $max_size * $n_oct;
				info("Le fichier uploader  dpass la taille maximal de $max_size octets\n");
				$size = 0;
		}
		# on recupere les delimiteur
    if( defined($q->param('feed_separator')) ) { $this->{SEPARATOR} = $this->{SEP_LIST}{$q->param('feed_separator')};}
    if( defined($q->param('feed_limit')) ) { $this->{TEXT} = $this->{DELIMITOR}{$q->param('feed_limit')};}

		return($size);
}

####################################################################
# sub Parse
#
#  fonction de parsing d'un fichier csv
#
# Parametre :
# $pattern : un hash contenant des regexp a appliquer
# sur chaque champ pour le valider
#
# Retour : une structure de type csv parsing
# voir Sitalibs Parsing
#################################################################### 
sub Parse {
    my ($this,$pattern) = @_;
    my $fh;
    my $csv_res = {};
    my $csv_struct = {};
    if(conduse("Sitalibs::Parsing_CSV::CSV_Parser"))
    {

				# la premiere ligne n'est jamais lu
				$csv_struct->{'file'}->{'header'} = 1;

				# nombre de colonnes
				my $n = @{$this->{KEYS}};
				$csv_struct->{'file'}->{'column'} = $n;
				debug("colonnes = $n\n") if ($DEBUG);
				# dlimiteur
				$csv_struct->{'file'}->{'separator'} = $this->{SEPARATOR};
				$csv_struct->{'file'}->{'text'} = $this->{TEXT};

				# mise en place du pattern et de la position pour chaque colonne
				for(my $k=0; $k<@{$this->{KEYS}}; $k++)
				{
						my $key = $this->{KEYS}->[$k];
						$csv_struct->{$key}->{'pattern'} = $pattern->{$key};
						$csv_struct->{$key}->{'position'} = $k;
				}

				# on parse le fichier temporaire
				open($fh,$this->{TMPFILE});
				CSV_Parser($fh,$csv_struct,$csv_res);
				close($fh);

				debug($csv_res->{'err'}." erreur(s)\n") if ($DEBUG);
				debug("accept count =".$csv_res->{'accept'}->{'count'}."\n") if ($DEBUG);
				debug("reject count =".$csv_res->{'reject'}->{'count'}."\n") if ($DEBUG);
				debug("invalid count =".$csv_res->{'invalid'}->{'count'}."\n") if ($DEBUG);
    }
    else
    {
				info("erreur de chargement sitalibs\n");
    }
    return($csv_res);
}




# affichage
# traite une strcture cvs_parsing
# doit contenir sur chaque ligne du h une clef feed
# qui prend comme valeur success ou une valeur parmis
# celle dans la table de correspondance  
# $ligne->{feed} = 'success';

sub Result($$$$,$)
{
    my ($this,$csv_res,$tbl_file,$tpl_res,$name_tbl) = @_;
    my $page = '';
		if(!defined($name_tbl))
		{
				$name_tbl = 'feed_result';
		}

    $this->{SUCCESS} = 0;

    if(conduse("Sitalibs::Config")
       && conduse("Sitalibs::Common::TemplateParse"))
    {
				my $tab = { titre=>undef,
										file_upload=>$this->{UPFILE},
										looplines=>[],
										error => 0 };
				my $looplines = [];
				# le titre du tableau de resultat
				# et les titres des colonnes
				my @titre = (@{$this->{KEYS}}, qw(result num_row orig));
				my $htitre = {};

				foreach my $k (@titre)
				{
						$htitre->{$k} = undef;
				}

				my $htab = Rewrite_Table($tbl_file,$name_tbl,$htitre);
				$tab->{titre} = $htab->{titre};

				for(my $n = 0; $n < @titre; $n++)
				{
						$tab->{@titre[$n]} = $htab->{valeurs}[$n]->{nom};
       	}

				# lignes invalides & valide
				foreach my $l (@{$csv_res->{'invalid'}->{'lines'}},
											 @{$csv_res->{'accept'}->{'lines'}})
				{
						my $lgn = {};

						# on place sur le href de ligne les donnes a afficher
						foreach my $keys (@{$this->{KEYS}})
						{
								$lgn->{$keys} = $l->{'parsed'}->{$keys};
						}
						# le resultat a afficher
						if(defined($l->{'result'}))
						{
								$lgn->{result} = $l->{'result'};
						}
						else
						{
								$lgn->{result} = 'invalid';
						}
						# on recupere le numero de ligne
						$lgn->{num_row} = $l->{num_row};
						# on recupere la ligne originale
						$lgn->{orig} = $l->{orig};
						# ligne en erreur
						if($lgn->{result}=~/success/)
						{
								$lgn->{line_error} = 0;
						}
						else
						{
								$lgn->{line_error} = 1;
						}
						# mise a dispo du code resultat
						$lgn->{value} = $lgn->{result};
						# indicateur de resultat
						$lgn->{$lgn->{result}} = 1;
						# conversion suivant la rewrite table
						my $htab = Rewrite_Table($tbl_file,$name_tbl,$lgn);
						for(my $i=0; $i<@titre;$i++)
						{
								$lgn->{@titre[$i]} = $htab->{valeurs}->[$i]->{valeur};
						}
						# dtermine si il y a au moin un success
						$this->{SUCCESS} = $this->{SUCCESS} || (!$lgn->{line_error});
						push(@$looplines,$lgn);
				}

				$tab->{looplines} = $looplines;
				# ecriture du template
				$page = remplace($tpl_res,$tab) || info("Erreur de parsing du template $tpl_res\n");
    }
    else
    {
				info("erreur de chargement sitalibs\n");
				$page = "erreur de chargement sitalibs";
    }
    return($page);
}

# conduse emprunt a hugues
# permet de charger un module a la demande
sub conduse($;$)
{
    my ($package, $tag) = @_;

    if (eval "require $package")
    {
				if (defined $tag)
				{
						$package->import($tag);
				}
				else
				{
						$package->import();
				}
				return 1;
    }
    info("[conduse] Package $package failed to import ($@)!\n");
    return 0;
}

1;
