# This file is part of the "FDN tools" project.
# You can use and redistribute it under the terms of the
# GNU General Public License.
# (c) French Data Network 1997-2005
# Authors and contributors:
# - Benjamin Bayart
# - Alexis Sukrieh
package Sitalibs::Web::SPOC;
#
#   $Id$
#   $Name$
#

use strict;
use warnings;

use Exporter;
use URI::Escape ;
use Sitalibs::Common::Logger;

#use Data::Dumper; # DEBUG

our ($VERSION, @ISA, @EXPORT) ;

$VERSION    = 1.0 ;
@ISA        = qw(Exporter) ;
@EXPORT     = qw(
	&check_protocol 
	&get_action 
	&get_parameters 
	&get_error 
	&get_response
	&get_file_response
);

##########
# Retourne l'action passe dans la requte HTTP et supprime la clef
# correspondante dans le hash contenant les paramtres de la requte..
#
# Arguments: 
#   - hash contenant les paramtres de la requte
#	{
#	    'param0'	=> 'val0',
#	    ...
#	    'paramN'	=> 'valN'
#	}
#   - table de hash contenant les actions existentes
#   - nom du paramtre contenant l'action  rcuperer
#
# Valeurs de retour:
#   - nom de l'action si tout s'est bien pass
#   - undef sinon
#  
sub get_action($$$)
{
    my ($query, $actions, $name) = @_ ;
    my $action ;

    unless ($query and $actions and $name) {
	return ;
    }
    
    $action =  $query->{$name} ;
    
    unless (defined $action and defined $actions->{$action}) {
	return ;
    }
    #
    # On supprime ce paramtre qui ne reservira plus dsormais et qui gnerait
    # surtout dans les tapes suivantes lors de la construction de notre
    # structure, de la vrification etc.
    #
    delete $query->{$name} ;
    
    return ($action) ;
}

##########
# Renvoie une structure contenant toutes les informations utiles
# de la requte
#
# Arguments:
#   - hash contenant les paramtres de la requte
#	{
#	    'param0'	=> 'val0',
#	    ...
#	    'paramN'	=> 'valN'
#	}
#   - nom de l'action (pas utilis pour le moment)
#   - refrence sur le hash de structure de donnes "template" 
#     servant  verifier la validit de la requte et  construire
#     notre structure  retourner
#	{
#	    'action'	=> {
#		'param0'    => 0,	paramtres optionnel
#		'param1'    => 1,	paramtre obligatoire
#		'param2'    => [],      liste d'lments
#		'param3'    => [	groupe de paramtres
#		    {
#			'attr0'	=> 1,
#			...
#			'attrN'	=> 1
#		    }
#		]
#	    }
#	}
#     
# Valeurs de retour:
#   - une rfrence sur notre structure utile si tout s'est bien pass
#	{
#	    'param0'	=> 'val0',
#	    'param1'	=> 'val1',
#	    'param2'	=> [ 'val20', ..., 'val2N' ],
#	    'param3'	=> [ 
#		{
#		    'attr00'	=> 'val00',
#		    ...
#		    'attrN0'	=> 'valN0'
#		},
#		...
#		{
#		    'attr0N'	=> 'val0N',
#		    ...
#		    'attrNN'	=> 'valNN'
#		}
#	    ]
#	}
#   - sinon renvoie un hash vide
#
sub get_parameters($$$)
{
    my ($query, $action, $struct) = @_ ;
    my $ret = {} ;

    #
    # on parcourt toutes les clefs de la structure  crer
    #
    foreach my $param (keys %$struct) {
	if (ref $struct->{$param} eq 'ARRAY') {
	    
	    $ret->{$param} = [] ;

	    if (ref $struct->{$param}->[0] eq 'HASH') {
		#
		# liste de groupes d'attributs
		#
		
	    	foreach my $key (keys %{$struct->{$param}->[0]}) {
		    my $i = 0 ;
		    
		    while (defined $query->{$key . $i}) {
			$ret->{$param}->[$i]->{$key} = $query->{$key . $i} ;
			delete $query->{$key . $i} ;
			$i += 1 ;
		    }
		}
	    }
	    else {
		#
		# liste de N lments (0 compris)
		#
                if (defined $query->{$param}) {
		    push @{$ret->{$param}}, $query->{$param} ;
		    delete $query->{$param} ;
                }
                elsif (defined $query->{$param . '0'}) {
		    my $i = 0 ;
		    
		    while (defined $query->{$param . $i}) {
			push @{$ret->{$param}}, $query->{$param . $i} ;
			delete $query->{$param . $i} ;
			$i += 1 ;
		    }
		}
		#
		# on ne renvoie pas undef car il se peut qu'il n'y ait
		# pas de param si num-param=0
		#
	    }
	}
	else {
	    #
	    # param unique (obligatoire ou optionnel)
	    #
	    if ($struct->{$param} == 1 or $struct->{$param} == 0) {
		if (not defined $query->{$param . '0'}) {
		    if (defined $query->{$param}) {
			$ret->{$param} = $query->{$param} ;
			delete $query->{$param} ;
		    }
		}
		else {
		    $ret->{$param} = $query->{$param . '0'} ;
		    delete $query->{$param . '0'} ;
		}
	    }
	}
    }
    #
    # arrivs ici on a tout "transvas" de $query vers $ret.
    # dans $query il ne reste plus que les param de nom erron
    # et les num-param 
    #
    return ($ret) ;
}

##########
# Vrifie la cohrence de la requte
#
# Arguments:
#   - hash contenant la structure retourne par get_parameters()
#   - hash contenant les paramtres passs dans la requte et non utiliss
#     dans la structure retourne par get_parameters() comme les num-attr,
#     ou les paramtres invalides
#   - refrence sur le hash de structure de donnes "template" 
#     servant  verifier la validit de la requte
#
# Valeurs de retour:
#   - 0 OK
#   - sinon une erreur
#   
sub check_protocol($$$)
{
    my ($query, $remain, $struct) = @_ ;
    
    #
    # comme dans $remain il reste tous les paramtre du type num-lala
    # on vrifie directement toutes les clefs de $remain pour verifier si
    # on a bien des valeurs numrique
    #
    foreach my $key (keys %$remain) {
	if ($key =~ m/^num-(.+)$/) {
	   if ($remain->{$key} !~ m/^\d+$/) {
		return (1) ;
	   }
	}
	else {
		debug "key : $key";
	    return (10) ;
	}
    }
    
    foreach my $param (keys %$struct) {
	if (ref $struct->{$param} eq 'ARRAY') {
	    if (ref $struct->{$param}->[0] eq 'HASH') {
		#
		# groupe de paramtres
		# 
		my $nb_group = (@{$query->{$param}}) ;
		
		foreach my $key (keys %{$struct->{$param}->[0]}) {
		    my $num_param = 1 ;
		    my $i = 0 ;

		    if (defined $remain->{"num-$key"}) {
			$num_param = $remain->{"num-$key"} ;
			delete $remain->{"num-$key"} ;
		    }
		    if ($num_param != $nb_group) {
			#
                        # return 2 si num-param > nb elements du groupe
                        # return 3 si num-param > nb elements du groupe
		        #
			return ($num_param > $nb_group ? 2 : 3) ;
		    }
		    while ($i < $nb_group) {
			if (not defined $query->{$param}->[$i]->{$key}) {
			    return (4) ;
			}
			$i += 1 ;
		    }
		}
	    }
	    else {
		#
		# N lments
		#
		my $num_param = 1 ;

		if (defined $remain->{"num-$param"}) {
		    $num_param = $remain->{"num-$param"} ;
		    delete $remain->{"num-$param"} ;
		}
		if ($num_param != (my $nb_param = @{$query->{$param}})) {
		    #
		    # return 5 si param < num-param 
		    # return 6 si param > num-param
		    #
		    return (($nb_param > $num_param) ? 5 : 6) ;
		}
	    }
	}
	else {
	    #
	    # lment unique (obligatoire ou optionnel)
	    #
	    if (defined $remain->{"num-$param"}) {
		if ($remain->{"num-$param"} != 1) {
		    #
                    # return 7 si num-param trop grand
                    # return 8 si num-param trop petit
                    #
		    return (($remain->{"num-$param"} > 1) ? 7 : 8) ;
		}
		delete $remain->{"num-$param"} ;
	    }
	    if (not defined $query->{$param}) {
		if ($struct->{$param} == 1) {
			info "argument obligatoire $param non fourni.";
			return 9;
		}
		else {
			debug "argument optionel $param non fourni";	
		}
	    }
	}
    }
    #
    # Si tout est ok, arriv ici, le hash $remain ne doit plus contenir
    # aucune clef. Si ce n'est celles correspondantes  des paramtres
    # invalides
    #
    if ((keys %$remain)) {
	debug "arguments non attendus fournis: " . join(", ", keys(%{$remain}));
	return (10) ;
    }
    
    return (0) ;
}

# permet de formater un fichier reu en base 64 selon le format SPOC.
# Tel que spcifi dans STD VoIP, rvision 1.13
sub get_file_response($;$)
{
	my ($filecontent, $filename, $ret) = ($_[0], $_[1], "");
	return "" unless defined $filecontent;
	$filename = "unknownfile" unless defined $filename;

	# Gestion du cas _file
	debug "Traitement d'une entre de type fichier";
	$filecontent =~ s/(.{80})/$1\n/g if 
		length $filecontent > 80;
	$ret .= "FILE $filename\n";
	$ret .= $filecontent."\n";
	$ret .= "EOF $filename\n";

	return $ret;
}

##########
# Permet de formater une rponse  une requte
#
# Arguments:
#   - hash contenant une structure semblable  celle retourne par
#     get_parameters()
#   - nom de l'action (n'est actuellement pas utilis, en prvision)
#
# Valeurs de retour:
#   - chaine contenant la requte  renvoyer, formate, contenant les \n etc.
#
# Attention:
#   - la gestion des erreurs doit tre faite avant d'utiliser cette
#     fonction.
sub get_response($$)
{
    my ($query, $action) = @_ ;
    my $ret = '' ;
    
    $ret = "OK\n" if keys %$query ;
	
    foreach my $param (keys %$query) {
	

	
        if (ref $query->{$param} eq 'ARRAY') {
	    if (ref $query->{$param}->[0] eq 'HASH') {
		#
		# liste de hash
		#
		# $nb groupes d'attributs
		#
		my $nb = @{$query->{$param}} ;
	    
		foreach my $attr (keys %{$query->{$param}->[0]}) {
			my $i = $nb ;			

			$ret .= "num-$attr=$nb\n" ;

			while ($i > 0) {
			    $i -= 1 ;
			    $ret .= "$attr$i=" .
				_uri_encode($query->{$param}->[$i]->{$attr}) 
				. "\n" ;
			}
		}
	    }
	    else {
		#
		# liste
		#
		my $nb = int(@{$query->{$param}});
		$ret .= "num-$param=$nb\n" ;

		while ($nb > 0) {
			$nb -= 1 ;	
			$ret .= "$param$nb="._uri_encode($query->{$param}->[$nb])."\n";
		}

	    }
	}
	else {
	    #
	    # paramtre unique
	    # donc pas de num-lala
	    #
	    $ret .= "$param=" . _uri_encode($query->{$param}) . "\n" ;
	}
    }

    return ($ret) ;
}

##########
# Permet d'chapper les caractres pouvant poser des problmes
# (utilisatio de URI::Escape)
#
# Arguments:
#   - chane de caractres
#
# Valeurs de retour:
#   - chane de caractres avec les bons caractres echapps
#
sub _uri_encode($) {
    my ($uri) = @_ ;

    unless (defined $uri) {
	return "";
    }
    return (uri_escape($uri)) ;
}


1;

